/**
 * @license
 * Copyright 2020 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @internal
 */
export const PUPPETEER_REVISIONS = Object.freeze({
  chrome: '139.0.7258.66',
  'chrome-headless-shell': '139.0.7258.66',
  firefox: 'stable_141.0.2',
});
